unit ocFileUtils;

interface
uses Windows,SysUtils,Classes, RTLConsts;
type

  TFileStreamWriteThrough = class (TFileStream)
  public
    constructor Create(const FileName: string; Mode: Word); overload;
    constructor Create(const FileName: string; Mode: Word; Rights: Cardinal); overload;
    destructor Destroy; override;
  end;

  TFileStreamUnbuffered = class (TFileStream)
  public
    constructor Create(const FileName: string; Mode: Word); overload;
    constructor Create(const FileName: string; Mode: Word; Rights: Cardinal); overload;
    destructor Destroy; override;
  end;

  TFileStreamRandomAccess = class (TFileStream)
  public
    constructor Create(const FileName: string; Mode: Word); overload;
    constructor Create(const FileName: string; Mode: Word; Rights: Cardinal); overload;
    destructor Destroy; override;
  end;

  TFileStreamSequentalScan = class (TFileStream)
  public
    constructor Create(const FileName: string; Mode: Word); overload;
    constructor Create(const FileName: string; Mode: Word; Rights: Cardinal); overload;
    destructor Destroy; override;
  end;

  //       
function CopyDir (Src,Dest:string; Subfolders:boolean; var WrongFile:string):boolean;
function SaveStringToFile (const FileName,Text:string):boolean;
function SaveStringStreamToFile(const StStr:TStringStream;const FileName:string):boolean;

implementation

function SaveStringStreamToFile(const StStr:TStringStream;const FileName:string):boolean;
var  fs:TFileStreamSequentalScan;
begin
  result:=false;
  if StStr=nil then exit;
  
  try
    fs:=TFileStreamSequentalScan.Create(FileName,fmCreate or fmShareDenyWrite);
  except
    exit;
  end;

  try
    StStr.Position:=0;
    fs.CopyFrom(StStr,StStr.Size);
  except
    exit;
  end;

  try
    FreeAndNil(fs);
  except
  end;

  result:=true;
end;

function SaveStringToFile (const FileName,Text:string):boolean;
var
  txtfile :TFileStreamSequentalScan;
begin
  result:=false;
  try
    txtfile:=TFileStreamSequentalScan.Create(FileName,fmCreate or fmShareDenyWrite);
  except
    exit;
  end;

  if Text<>'' then
  try
    txtfile.WriteBuffer(Text[1],system.length(Text));
  except
    exit;
  end;

  try
    FreeAndNil(txtfile);
  except
  end;
  result:=true;
end;

function CopyDir (Src,Dest:string; Subfolders:boolean; var WrongFile:string):boolean;

  function CopyFile (const Src,Dest:string; var WrongFile:string):boolean;
  var FS,FD:TFileStreamSequentalScan;
  begin
    WrongFile:='';
    result:=false;
    try
      FS:=TFileStreamSequentalScan.Create(Src,fmOpenRead or fmShareDenyWrite);
    except
      WrongFile:=Src;
      exit;
    end;
    try
      FD:=TFileStreamSequentalScan.Create(Dest,fmCreate or fmShareExclusive);
    except
      WrongFile:=Dest;
      exit;
    end;

    try
      FD.CopyFrom(FS,FS.Size);
    except
      WrongFile:=Dest;
      exit;
    end;

    try FD.Free except end;
    try FS.Free except end;

    result:=true;
  end;

  function CopyOneFolder (const Src,Dest:string;var WrongFile:string):boolean;
  var
    sr: TSearchRec;
  begin
    result:=false;
    if not ForceDirectories(Dest) then exit;
    
    if FindFirst(Src+'*.*', faAnyFile, sr) = 0 then
    begin
      repeat
        if (sr.Attr and faDirectory) = faDirectory then
        begin
          if Subfolders and (sr.Name<>'.') and (sr.Name<>'..') then
            result:=CopyOneFolder(Src+sr.Name+'\',Dest+sr.Name+'\',WrongFile) else
            result:=true;
        end    
        else
          result:=CopyFile(Src+sr.Name,Dest+sr.Name,WrongFile);

        if not result then exit;
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;

  end;

begin
  result:=false;

  //normalize
  while (Src[Length(Src)]='\') do Delete(Src,Length(Src),1);
  while (Dest[Length(Dest)]='\') do Delete(Dest,Length(Dest),1);
  //checking...
  if (Src='') or (not DirectoryExists(Src)) then
  begin
    WrongFile:=Src;
    exit;
  end;
  if (Dest='') then
  begin
    WrongFile:='<<empty destination>>';
    exit;
  end;

  Src:=Src+'\';
  Dest:=Dest+'\';
  if ForceDirectories(Dest) then
    result:=CopyOneFolder(Src,Dest,WrongFile);

end;

function FileOpen(const FileName: string; Mode: LongWord; const AdvancedFlags:Cardinal): Integer;
const
  AccessMode: array[0..2] of LongWord = (
    GENERIC_READ,
    GENERIC_WRITE,
    GENERIC_READ or GENERIC_WRITE);
  ShareMode: array[0..4] of LongWord = (
    0,
    0,
    FILE_SHARE_READ,
    FILE_SHARE_WRITE,
    FILE_SHARE_READ or FILE_SHARE_WRITE);
begin
{$Warnings off}
result:=INVALID_HANDLE_VALUE;
{$Warnings on}
  if ((Mode and $F0) shr 4) and  FILE_SHARE_WRITE = FILE_SHARE_WRITE then
    if FileExists(FileName) and FileIsReadOnly(FileName) then
      if not FileSetReadOnly(FileName,false) then
        exit;

  result := Integer(CreateFile(PChar(FileName), AccessMode[Mode and 3],
      ShareMode[(Mode and $F0) shr 4], nil, OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL or AdvancedFlags, 0));
end;

function FileCreate(const FileName: string; const AdvancedFlags:Cardinal): Integer;
begin
{$Warnings off}
result:=INVALID_HANDLE_VALUE;
{$Warnings on}
  try
    if FileExists(FileName) and FileIsReadOnly(FileName) then
      if not FileSetReadOnly(FileName,false) then
        exit;
  except
  end;

  result := Integer(CreateFile(PChar(FileName), GENERIC_READ or GENERIC_WRITE,
      FILE_SHARE_READ, nil, CREATE_ALWAYS,
      FILE_ATTRIBUTE_NORMAL or AdvancedFlags, 0));
end;

{ TFileStreamWriteThrough }

constructor TFileStreamWriteThrough.Create(const FileName: string;
  Mode: Word);
begin
  Create(Filename, Mode, 0);
end;

constructor TFileStreamWriteThrough.Create(const FileName: string;
  Mode: Word; Rights: Cardinal);
begin
  if Mode = fmCreate then
  begin
    inherited Create(FileCreate(FileName, FILE_FLAG_WRITE_THROUGH));
    if FHandle < 0 then
      raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
  end
  else
  begin
    inherited Create(FileOpen(FileName, Mode, FILE_FLAG_WRITE_THROUGH));
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
  end;
end;

destructor TFileStreamWriteThrough.Destroy;
begin

  inherited;
end;

{ TFileStreamUnbuffered }

constructor TFileStreamUnbuffered.Create(const FileName: string;
  Mode: Word);
begin
  Create(Filename, Mode, 0);
end;

constructor TFileStreamUnbuffered.Create(const FileName: string;
  Mode: Word; Rights: Cardinal);
begin
  if Mode = fmCreate then
  begin
    inherited Create(FileCreate(FileName, FILE_FLAG_NO_BUFFERING));
    if FHandle < 0 then
      raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
  end
  else
  begin
    inherited Create(FileOpen(FileName, Mode, FILE_FLAG_NO_BUFFERING));
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
  end;
end;

destructor TFileStreamUnbuffered.Destroy;
begin

  inherited;
end;

{ TFileStreamRandomAccess }

constructor TFileStreamRandomAccess.Create(const FileName: string;
  Mode: Word);
begin
  Create(Filename, Mode, 0);
end;

constructor TFileStreamRandomAccess.Create(const FileName: string;
  Mode: Word; Rights: Cardinal);
begin
  if Mode = fmCreate then
  begin
    inherited Create(FileCreate(FileName, FILE_FLAG_RANDOM_ACCESS));
    if FHandle < 0 then
      raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
  end
  else
  begin
    inherited Create(FileOpen(FileName, Mode, FILE_FLAG_RANDOM_ACCESS));
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
  end;
end;

destructor TFileStreamRandomAccess.Destroy;
begin

  inherited;
end;

{ TFileStreamSequentalScan }

constructor TFileStreamSequentalScan.Create(const FileName: string;
  Mode: Word);
begin
  Create(Filename, Mode, 0);
end;

constructor TFileStreamSequentalScan.Create(const FileName: string;
  Mode: Word; Rights: Cardinal);
begin
  if Mode = fmCreate then
  begin
    inherited Create(FileCreate(FileName, FILE_FLAG_SEQUENTIAL_SCAN));
    if FHandle < 0 then
      raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
  end
  else
  begin
    inherited Create(FileOpen(FileName, Mode, FILE_FLAG_SEQUENTIAL_SCAN));
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
  end;
end;

destructor TFileStreamSequentalScan.Destroy;
begin

  inherited;
end;

end.
